"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_startcase_1 = __importDefault(require("lodash.startcase"));
/**
 * Codemod to update icon imports
 * ```
 * npx jscodeshift -t node_modules/evergreen-ui/codemods/dist/4.28.1-4.29.0/replace-icon-imports.js --parser=tsx --extensions=js,ts,tsx <your file target> --dry --print
 * ```
 *
 * Converts:
 * <Icon icon="foo" /> to <FooIcon />
 * <IconButton icon="foo" /> to <IconButton icon={FooIcon} />
 * <Button iconAfter="foo" /> to <Button iconAfter={FooIcon} />
 * ... and more
 */
function matchStringLiteralIconProps(attribute, prop = 'icon') {
    var _a, _b;
    return ((_b = (attribute.type === 'JSXAttribute' && attribute.name.name === prop && ((_a = attribute.value) === null || _a === void 0 ? void 0 : _a.type) === 'Literal')) !== null && _b !== void 0 ? _b : false);
}
const ElementProps = {
    IconButton: { props: ['icon', 'iconBefore', 'iconAfter'], componentName: 'IconButton' },
    Button: { props: ['iconBefore', 'iconAfter'], componentName: 'Button' },
    BackButton: { props: ['iconBefore', 'iconAfter'], componentName: 'BackButton' },
    TextDropdownButton: { props: ['icon'], componentName: 'TextDropdownButton' },
    SearchTableHeaderCell: { props: ['icon'], componentName: 'SearchTableHeaderCell' },
    Table: { props: ['icon'], componentName: 'Table.SearchHeaderCell' },
    SelectMenu: { props: ['filterIcon'], componentName: 'SelectMenu' },
    SelectMenuContent: { props: ['filterIcon'], componentName: 'SelectMenuContent' },
    OptionsList: { props: ['filterIcon'], componentName: 'OptionsList' },
    Menu: { props: ['icon'], componentName: 'Menu.Item' },
    ListItem: { props: ['icon'], componentName: 'ListItem' },
    UnorderedList: { props: ['icon'], componentName: 'UnorderedList' },
    Ul: { props: ['icon'], componentName: 'Ul' },
    OrderedList: { props: ['icon'], componentName: 'OrderedList' },
    Ol: { props: ['icon'], componentName: 'Ol' },
};
const transform = (file, api) => {
    // Alias the jscodeshift API for ease of use.
    const j = api.jscodeshift;
    // Convert the entire file source into a collection of nodes paths.
    const root = j(file.source);
    const evergreenImports = root
        .find(j.ImportDeclaration, {
        source: {
            value: 'evergreen-ui',
        },
    })
        .nodes();
    if (evergreenImports.length === 0) {
        return;
    }
    // Assuming there are no duplicate import declarations
    const evergreenImport = evergreenImports[0];
    // Find Icon components
    root
        .findJSXElements('Icon')
        /* @ts-ignore */
        .filter((path) => path.node.openingElement.attributes.some((a) => matchStringLiteralIconProps(a, 'icon')))
        .forEach(({ value: Node }) => {
        // Swap the component name
        /* @ts-ignore */
        const iconProp = Node.openingElement.attributes.find((a) => matchStringLiteralIconProps(a, 'icon'));
        if (!iconProp) {
            return;
        }
        /* @ts-ignore */
        const icon = iconProp.value.value;
        if (!icon) {
            return;
        }
        const iconName = (0, lodash_startcase_1.default)(icon).replace(/ /g, '');
        const componentName = `${iconName}Icon`;
        Node.openingElement.name = j.jsxIdentifier(componentName);
        // Replace the import path too
        /* @ts-ignore */
        const hasImportAlready = evergreenImport.specifiers.some((s) => s.imported.name === componentName);
        if (!hasImportAlready) {
            /* @ts-ignore */
            evergreenImport.specifiers.push(j.importSpecifier(j.identifier(componentName)));
        }
    })
        // Remove the `icon` prop
        .find(j.JSXAttribute, {
        name: {
            type: 'JSXIdentifier',
            name: 'icon',
        },
        value: {
            type: 'Literal',
        },
    })
        .remove();
    // For all other components, find any icon-receiving props
    for (const { componentName, props } of Object.values(ElementProps)) {
        const [openingElement, identifier] = componentName.split('.');
        let components = root.findJSXElements(openingElement);
        if (identifier) {
            components = root.find(j.JSXElement, {
                openingElement: {
                    name: {
                        property: {
                            type: 'JSXIdentifier',
                            name: identifier,
                        },
                    },
                },
            });
        }
        for (const prop of props) {
            components
                .find(j.JSXAttribute, {
                name: {
                    type: 'JSXIdentifier',
                    name: prop,
                },
                value: {
                    type: 'Literal',
                },
            })
                .find(j.Literal)
                .replaceWith((nodePath) => {
                const { node } = nodePath;
                // e.g. "add" -> "AddIcon"
                const iconName = `${(0, lodash_startcase_1.default)(node.value).replace(/ /g, '')}Icon`;
                // Replace the import path too
                /* @ts-ignore */
                const hasImportAlready = evergreenImport.specifiers.some((s) => s.imported.name === iconName);
                if (!hasImportAlready) {
                    /* @ts-ignore */
                    evergreenImport.specifiers.push(j.importSpecifier(j.identifier(iconName)));
                }
                // with a new JSX expression with the icon definition.
                return j.jsxExpressionContainer(j.identifier(iconName));
            });
        }
    }
    // Remove `Icon` from imports if now unused
    const remainingIcons = root.findJSXElements('Icon');
    if (remainingIcons.size() === 0) {
        root
            .find(j.ImportDeclaration, {
            source: {
                value: 'evergreen-ui',
            },
        })
            .find(j.ImportSpecifier, {
            imported: {
                name: 'Icon',
            },
        })
            .remove();
    }
    return root.toSource();
};
exports.default = transform;
